@@ -14,6 +14,7 @@ import os
 import time
 import unittest
 import sys
+import decimal
 
 import jsonpickle
 
@@ -26,6 +27,8 @@ from jsonpickle.compat import PY32
 from jsonpickle._samples import (
         BrokenReprThing,
         DictSubclass,
+        GetstateDict,
+        GetstateReturnsList,
         ListSubclass,
         ListSubclassWithInit,
         NamedTuple,
@@ -43,6 +46,10 @@ class PicklingTestCase(unittest.TestCase):
         self.pickler = jsonpickle.pickler.Pickler()
         self.unpickler = jsonpickle.unpickler.Unpickler()
 
+    def tearDown(self):
+        self.pickler.reset()
+        self.unpickler.reset()
+
     def test_string(self):
         self.assertEqual('a string', self.pickler.flatten('a string'))
         self.assertEqual('a string', self.unpickler.restore('a string'))
@@ -221,6 +228,27 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual(old_nt[1], new_nt[1])
         self.assertEqual(old_nt[2], new_nt[2])
 
+    def test_counter_roundtrip(self):
+        if sys.version_info < (2, 7):
+            # collections.Counter was introduced in Python 2.7
+            return
+        counter = collections.Counter({1: 2})
+        encoded = jsonpickle.encode(counter)
+        decoded = jsonpickle.decode(encoded)
+        self.assertTrue(type(decoded) is collections.Counter)
+        # the integer key becomes a string when keys=False
+        self.assertEqual(decoded.get('1'), 2)
+
+    def test_counter_roundtrip_with_keys(self):
+        if sys.version_info < (2, 7):
+            # collections.Counter was introduced in Python 2.7
+            return
+        counter = collections.Counter({1: 2})
+        encoded = jsonpickle.encode(counter, keys=True)
+        decoded = jsonpickle.decode(encoded, keys=True)
+        self.assertTrue(type(decoded) is collections.Counter)
+        self.assertEqual(decoded.get(1), 2)
+
     def test_class(self):
         inst = Thing('test name')
         inst.child = Thing('child name')
@@ -359,6 +387,70 @@ class PicklingTestCase(unittest.TestCase):
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(1, inflated['key1'])
 
+    def test_getstate_dict_subclass_structure(self):
+        obj = GetstateDict('test')
+        obj['key1'] = 1
+
+        flattened = self.pickler.flatten(obj)
+        self.assertTrue(tags.OBJECT in flattened)
+        self.assertEqual('jsonpickle._samples.GetstateDict',
+                         flattened[tags.OBJECT])
+        self.assertTrue(tags.STATE in flattened)
+        self.assertTrue(tags.TUPLE in flattened[tags.STATE])
+        self.assertEqual(['test', {'key1': 1}],
+                         flattened[tags.STATE][tags.TUPLE])
+
+    def test_getstate_dict_subclass_roundtrip_simple(self):
+        obj = GetstateDict('test')
+        obj['key1'] = 1
+
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+
+        self.assertEqual(1, inflated['key1'])
+        self.assertEqual(inflated.name, 'test')
+
+    def test_getstate_dict_subclass_roundtrip_cyclical(self):
+        obj = GetstateDict('test')
+        obj['key1'] = 1
+
+        # The "name" field of obj2 points to obj (reference)
+        obj2 = GetstateDict(obj)
+        # The "obj2" key in obj points to obj2 (cyclical reference)
+        obj['obj2'] = obj2
+
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+
+        # The dict must be preserved
+        self.assertEqual(1, inflated['key1'])
+
+        # __getstate__/__setstate__ must have been run
+        self.assertEqual(inflated.name, 'test')
+        self.assertEqual(inflated.active, True)
+        self.assertEqual(inflated['obj2'].active, True)
+
+        # The reference must be preserved
+        self.assertTrue(inflated is inflated['obj2'].name)
+
+    def test_getstate_list_simple(self):
+        obj = GetstateReturnsList(1, 2)
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(inflated.x, 1)
+        self.assertEqual(inflated.y, 2)
+
+    def test_getstate_list_inside_list(self):
+        obj1 = GetstateReturnsList(1, 2)
+        obj2 = GetstateReturnsList(3, 4)
+        obj = [obj1, obj2]
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(inflated[0].x, 1)
+        self.assertEqual(inflated[0].y, 2)
+        self.assertEqual(inflated[1].x, 3)
+        self.assertEqual(inflated[1].y, 4)
+
     def test_tuple_notunpicklable(self):
         self.pickler.unpicklable = False
 
@@ -427,6 +519,14 @@ class PicklingTestCase(unittest.TestCase):
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(inflated.themodule, os)
 
+    def test_thing_with_module_safe(self):
+        obj = Thing('with-module')
+        obj.themodule = os
+        flattened = self.pickler.flatten(obj)
+        self.unpickler.safe = True
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(inflated.themodule, None)
+
     def test_thing_with_submodule(self):
         from distutils import sysconfig
 
@@ -487,6 +587,31 @@ class PicklingTestCase(unittest.TestCase):
         for x in range(len(coll)):
             self.assertEqual(repr(coll[x]), repr(inflated[x]))
 
+    def test_references_in_number_keyed_dict(self):
+        """
+        Make sure a dictionary with numbers as keys and objects as values
+        can make the round trip.
+
+        Because JSON must coerce integers to strings in dict keys, the sort
+        order may have a tendency to change between pickling and unpickling,
+        and this could affect the object references.
+        """
+        one = Thing('one')
+        two = Thing('two')
+        twelve = Thing('twelve')
+        two.child = twelve
+        obj = {
+            1: one,
+            2: two,
+            12: twelve,
+        }
+        self.assertNotEqual(list(sorted(obj.keys())),
+                            list(map(int, sorted(map(str, obj.keys())))))
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(len(inflated), 3)
+        self.assertEqual(inflated['12'].name, 'twelve')
+
     def test_list_subclass_with_init(self):
         obj = ListSubclassWithInit('foo')
         self.assertEqual(obj.attr, 'foo')
@@ -501,6 +626,12 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual(expect, actual)
         self.assertTrue(expect is actual)
 
+    def test_decimal(self):
+        obj = decimal.Decimal(1)
+        flattened = self.pickler.flatten(obj)
+        inflated = self.unpickler.restore(flattened)
+        self.assertEqual(type(inflated), decimal.Decimal)
+
 
 class JSONPickleTestCase(unittest.TestCase):
 
